@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -145,6 +143,7 @@
 #include <CoreFoundation/CFMachPort.h>
 #include <CoreFoundation/CFRunLoop.h>
 #include <CoreFoundation/CFDictionary.h>
+#include <CoreFoundation/CFByteOrder.h>
 #include <mach/mach.h>
 #include <mach/mach_error.h>
 #include <mach/notify.h>
@@ -268,51 +267,57 @@ void _CFMachPortInstallNotifyPort(CFRunLoopRef rl, CFStringRef mode) {
 static void __CFNotifyDeadMachPort(CFMachPortRef port, void *msg, CFIndex size, void *info) {
     mach_msg_header_t *header = (mach_msg_header_t *)msg;
     if (header && header->msgh_id == MACH_NOTIFY_DEAD_NAME) {
-	mach_port_t dead_port = ((mach_dead_name_notification_t *)msg)->not_port;
-	CFMachPortRef existing;
-	/* If the CFMachPort has already been invalidated, it won't be found here. */
-	__CFSpinLock(&__CFAllMachPortsLock);
-	if (NULL != __CFAllMachPorts && CFDictionaryGetValueIfPresent(__CFAllMachPorts, (void *)dead_port, (const void **)&existing)) {
-	    CFDictionaryRemoveValue(__CFAllMachPorts, (void *)dead_port);
-	    CFRetain(existing);
-	    __CFSpinUnlock(&__CFAllMachPortsLock);
-	    CFMachPortInvalidate(existing);
-	    CFRelease(existing);
-	} else {
-	    __CFSpinUnlock(&__CFAllMachPortsLock);
-	}
-	/* Delete port reference we got for this notification */
-	mach_port_deallocate(mach_task_self(), dead_port);
+		mach_port_t dead_port = ((mach_dead_name_notification_t *)msg)->not_port;
+		if (((mach_dead_name_notification_t *)msg)->NDR.int_rep != NDR_record.int_rep) {
+			dead_port = CFSwapInt32(dead_port);	
+		}
+		CFMachPortRef existing;
+		/* If the CFMachPort has already been invalidated, it won't be found here. */
+		__CFSpinLock(&__CFAllMachPortsLock);
+		if (NULL != __CFAllMachPorts && CFDictionaryGetValueIfPresent(__CFAllMachPorts, (void *)dead_port, (const void **)&existing)) {
+			CFDictionaryRemoveValue(__CFAllMachPorts, (void *)dead_port);
+			CFRetain(existing);
+			__CFSpinUnlock(&__CFAllMachPortsLock);
+			CFMachPortInvalidate(existing);
+			CFRelease(existing);
+		} else {
+			__CFSpinUnlock(&__CFAllMachPortsLock);
+		}
+		/* Delete port reference we got for this notification */
+		mach_port_deallocate(mach_task_self(), dead_port);
     } else if (header && header->msgh_id == MACH_NOTIFY_PORT_DELETED) {
-	mach_port_t dead_port = ((mach_port_deleted_notification_t *)msg)->not_port;
-	CFMachPortRef existing;
-	/* If the CFMachPort has already been invalidated, it won't be found here. */
-	__CFSpinLock(&__CFAllMachPortsLock);
-	if (NULL != __CFAllMachPorts && CFDictionaryGetValueIfPresent(__CFAllMachPorts, (void *)dead_port, (const void **)&existing)) {
-	    CFDictionaryRemoveValue(__CFAllMachPorts, (void *)dead_port);
-	    CFRetain(existing);
-	    __CFSpinUnlock(&__CFAllMachPortsLock);
-	    CFMachPortInvalidate(existing);
-	    CFRelease(existing);
-	} else {
-	    __CFSpinUnlock(&__CFAllMachPortsLock);
-	}
-	/* Delete port reference we got for this notification */
-// Don't do this, since this always fails, and could cause trouble
-//	mach_port_deallocate(mach_task_self(), dead_port);
+		mach_port_t dead_port = ((mach_port_deleted_notification_t *)msg)->not_port;
+		if (((mach_dead_name_notification_t *)msg)->NDR.int_rep != NDR_record.int_rep) {
+			dead_port = CFSwapInt32(dead_port);	
+		}
+		CFMachPortRef existing;
+		/* If the CFMachPort has already been invalidated, it won't be found here. */
+		__CFSpinLock(&__CFAllMachPortsLock);
+		if (NULL != __CFAllMachPorts && CFDictionaryGetValueIfPresent(__CFAllMachPorts, (void *)dead_port, (const void **)&existing)) {
+			CFDictionaryRemoveValue(__CFAllMachPorts, (void *)dead_port);
+			CFRetain(existing);
+			__CFSpinUnlock(&__CFAllMachPortsLock);
+			CFMachPortInvalidate(existing);
+			CFRelease(existing);
+		} else {
+			__CFSpinUnlock(&__CFAllMachPortsLock);
+		}
+		/* Delete port reference we got for this notification */
+		// Don't do this, since this always fails, and could cause trouble
+		//	mach_port_deallocate(mach_task_self(), dead_port);
     }
 }
 
 static Boolean __CFMachPortEqual(CFTypeRef cf1, CFTypeRef cf2) {
     CFMachPortRef mp1 = (CFMachPortRef)cf1;
     CFMachPortRef mp2 = (CFMachPortRef)cf2;
-    __CFMachPortCheckForFork();
+//    __CFMachPortCheckForFork();  do not do this here
     return (mp1->_port == mp2->_port);
 }
 
 static CFHashCode __CFMachPortHash(CFTypeRef cf) {
     CFMachPortRef mp = (CFMachPortRef)cf;
-    __CFMachPortCheckForFork();
+//    __CFMachPortCheckForFork();  do not do this here -- can cause strange reentrancies 3843642
     return (CFHashCode)mp->_port;
 }
 
@@ -455,11 +460,11 @@ CFMachPortRef CFMachPortCreateWithPort(CFAllocatorRef allocator, mach_port_t por
     __CFMachPortSetValid(memory);
     memory->_callout = callout;
     if (NULL != context) {
-	memmove(&memory->_context, context, sizeof(CFMachPortContext));
+	CF_WRITE_BARRIER_MEMMOVE(&memory->_context, context, sizeof(CFMachPortContext));
 	memory->_context.info = context->retain ? (void *)context->retain(context->info) : context->info;
     }
     if (NULL == __CFAllMachPorts) {
-	__CFAllMachPorts = CFDictionaryCreateMutable(kCFAllocatorSystemDefault, 0, NULL, NULL);
+	__CFAllMachPorts = CFDictionaryCreateMutable(kCFAllocatorMallocZone, 0, NULL, NULL); // XXX_PCB make it GC weak.
 	_CFDictionarySetCapacity(__CFAllMachPorts, 20);
     }
     CFDictionaryAddValue(__CFAllMachPorts, (void *)port, memory);
@@ -496,7 +501,7 @@ void CFMachPortGetContext(CFMachPortRef mp, CFMachPortContext *context) {
     __CFGenericValidateType(mp, __kCFMachPortTypeID);
     CFAssert1(0 == context->version, __kCFLogAssertion, "%s(): context version not initialized to 0", __PRETTY_FUNCTION__);
     __CFMachPortCheckForFork();
-    memmove(context, &mp->_context, sizeof(CFMachPortContext));
+    CF_WRITE_BARRIER_MEMMOVE(context, &mp->_context, sizeof(CFMachPortContext));
 }
 
 void CFMachPortInvalidate(CFMachPortRef mp) {
@@ -592,6 +597,7 @@ void CFMachPortInvalidateAll(void) {
 // it was a very bad idea to call it.
 }
 
+    
 static mach_port_t __CFMachPortGetPort(void *info) {
     CFMachPortRef mp = info;
     __CFMachPortCheckForFork();
@@ -628,6 +634,10 @@ CFRunLoopSourceRef CFMachPortCreateRunLoopSource(CFAllocatorRef allocator, CFMac
     __CFGenericValidateType(mp, __kCFMachPortTypeID);
     __CFMachPortCheckForFork();
     __CFMachPortLock(mp);
+    if (!__CFMachPortIsValid(mp)) {
+        __CFMachPortUnlock(mp);
+        return NULL;
+    }
 #if 0
 #warning CF: adding ref to receive right is disabled for now -- doesnt work in 1F
     if (!__CFMachPortHasReceive(mp)) {
